// Generated by CoffeeScript 1.8.0
(function() {
  var PDF, Stream, assert, childprocess, fs, path, phantomjs;

  fs = require('fs');

  Stream = require('stream').Readable;

  childprocess = require('child_process');

  path = require('path');

  assert = require('assert');

  phantomjs = require('phantomjs');

  module.exports = PDF = (function() {
    function PDF(html, options) {
      this.html = html;
      this.options = options != null ? options : {};
      if (this.options.script) {
        this.script = path.normalize(this.options.script);
      } else {
        this.script = path.join(__dirname, 'scripts', 'pdf_a4_portrait.js');
      }
      if (this.options.filename) {
        this.options.filename = path.resolve(this.options.filename);
      }
      assert(typeof this.html === 'string' && this.html.length, "html-pdf: Can't create a pdf without an html string");
    }

    PDF.prototype.toBuffer = function(callback) {
      return this.exec(function(err, res) {
        if (err) {
          return callback(err);
        }
        return fs.readFile(res.filename, function(err, buffer) {
          if (err) {
            return callback(err);
          }
          return fs.unlink(res.filename, function(err) {
            if (err) {
              return callback(err);
            }
            return callback(null, buffer);
          });
        });
      });
    };

    PDF.prototype.toStream = function(callback) {
      return this.exec(function(err, res) {
        var stream;
        if (err) {
          return callback(err);
        }
        try {
          stream = fs.createReadStream(res.filename);
        } catch (_error) {
          err = _error;
          return callback(err);
        }
        stream.on('end', function() {
          return fs.unlink(res.filename, function(err) {
            if (err) {
              return console.log('html-pdf:', err);
            }
          });
        });
        return callback(null, stream);
      });
    };

    PDF.prototype.toFile = function(filename, callback) {
      assert(arguments.length > 0, 'html-pdf: The method .toFile([filename, ]callback) requires a callback.');
      if (filename instanceof Function) {
        callback = filename;
        filename = void 0;
      } else {
        this.options.filename = path.resolve(filename);
      }
      return this.exec(callback);
    };

    PDF.prototype.exec = function(callback) {
      var child, stderr, stdout, timeout;
      child = childprocess.spawn(phantomjs.path, [this.script]);
      stdout = [];
      stderr = [];
      timeout = setTimeout(function() {
        child.stdin.end();
        child.kill();
        if (!stderr.length) {
          return stderr = [new Buffer('html-pdf: PDF generation timeout. Phantom.js script did not exit.')];
        }
      }, parseInt(this.options.timeout) || 30000);
      child.stdout.on('data', function(buffer) {
        return stdout.push(buffer);
      });
      child.stderr.on('data', function(buffer) {
        stderr.push(buffer);
        child.stdin.end();
        return child.kill();
      });
      child.on('exit', function(code) {
        var data, err, _ref;
        clearTimeout(timeout);
        if (code || stderr.length) {
          err = new Error(Buffer.concat(stderr).toString() || 'html-pdf: Unknown Error');
          return callback(err);
        } else {
          try {
            data = (_ref = Buffer.concat(stdout).toString()) != null ? _ref.trim() : void 0;
            data = JSON.parse(data);
          } catch (_error) {
            err = _error;
            return callback(err);
          }
          return callback(null, data);
        }
      });
      return child.stdin.write(JSON.stringify({
        html: this.html,
        options: this.options
      }) + '\n', 'utf8');
    };

    return PDF;

  })();

}).call(this);
